@@ -1,7 +1,13 @@
+__title__ = 'purl'
+__version__ = '0.4.1'
+__author__ = 'David Winterbottom'
+__license__ = 'MIT'
+
 import urlparse
 import urllib
 from collections import namedtuple
 
+
 # To minimise memory consumption, we use a namedtuple to store all instance
 # variables, as well as using the __slots__ attribute.
 _URLTuple = namedtuple("_URLTuple", "host username password scheme port path query fragment")
@@ -34,6 +40,22 @@ def parse(url_str):
 
 
 class URL(object):
+    """
+    The constructor can be used in two ways:
+
+    1. Pass a URL string::
+
+        >>> URL('http://www.google.com/search?q=testing')
+        u'http://google.com/search?q=testing'
+
+    2. Pass keyword arguments::
+
+        >>> URL(host='www.google.com', path='/search', query='q=testing')
+        u'http://google.com/search?q=testing'
+
+    If you pass both a URL string and keyword args, then the values of keyword
+    args take precedence.
+    """
 
     __slots__ = ("_tuple",)
 
@@ -102,6 +124,9 @@ class URL(object):
     # extra args are passed
 
     def netloc(self):
+        """
+        Return the netloc
+        """
         url = self._tuple
         if url.username and url.password:
             netloc = u'%s:%s@%s' % (url.username, url.password, url.host)
@@ -112,6 +137,11 @@ class URL(object):
         return netloc
 
     def host(self, value=None):
+        """
+        Return the host
+
+        :param string value: new host string
+        """
         if value:
             return URL._mutate(self, host=value)
         return self._tuple.host
@@ -119,17 +149,36 @@ class URL(object):
     domain = host
 
     def username(self):
+        """
+        Return the username
+        """
         return self._tuple.username
 
     def password(self):
+        """
+        Return the password
+        """
         return self._tuple.password
 
     def subdomains(self, value=None):
+        """
+        Returns a list of subdomains or set the subdomains and returns a
+        new :class:`URL` instance.
+
+        :param list value: a list of subdomains
+        """
         if value is not None:
             return URL._mutate(self, host='.'.join(value))
         return self.host().split('.')
 
     def subdomain(self, index, value=None):
+        """
+        Return a subdomain or set a new value and return a new :class:`URL`
+        instance.
+
+        :param integer index: 0-indexed subdomain
+        :param string value: New subdomain
+        """
         if value is not None:
             subdomains = self.subdomains()
             subdomains[index] = value
@@ -137,11 +186,23 @@ class URL(object):
         return self.subdomains()[index]
 
     def scheme(self, value=None):
+        """
+        Return or set the scheme.
+
+        :param string value: the new scheme to use
+        :returns: string or new :class:`URL` instance
+        """
         if value:
             return URL._mutate(self, scheme=value)
         return self._tuple.scheme
 
     def path(self, value=None):
+        """
+        Return or set the path
+
+        :param string value: the new path to use
+        :returns: string or new :class:`URL` instance
+        """
         if value:
             if not value.startswith('/'):
                 value = '/' + value
@@ -149,16 +210,34 @@ class URL(object):
         return self._tuple.path
 
     def query(self, value=None):
+        """
+        Return or set the query string
+
+        :param string value: the new query string to use
+        :returns: string or new :class:`URL` instance
+        """
         if value:
             return URL._mutate(self, query=value)
         return self._tuple.query
 
     def port(self, value=None):
+        """
+        Return or set the port
+
+        :param string value: the new port to use
+        :returns: string or new :class:`URL` instance
+        """
         if value:
             return URL._mutate(self, port=value)
         return self._tuple.port
 
     def fragment(self, value=None):
+        """
+        Return or set the fragment (hash)
+
+        :param string value: the new fragment to use
+        :returns: string or new :class:`URL` instance
+        """
         if value:
             return URL._mutate(self, fragment=value)
         return self._tuple.fragment
@@ -166,6 +245,10 @@ class URL(object):
     def path_segment(self, index, value=None, default=None):
         """
         Return the path segment at the given index
+        
+        :param integer index: 
+        :param string value: the new segment value
+        :param string default: the default value to return if no path segment exists with the given index
         """
         if value is not None:
             segments = list(self.path_segments())
@@ -180,6 +263,11 @@ class URL(object):
             return default
 
     def path_segments(self, value=None):
+        """
+        Return the path segments
+        
+        :param list value: the new path segments to use
+        """
         if value is not None:
             new_path = '/' + '/'.join(value)
             return URL._mutate(self, path=new_path)
@@ -190,18 +278,44 @@ class URL(object):
         return tuple(segments)
 
     def add_path_segment(self, value):
+        """
+        Add a new path segment to the end of the current string
+        
+        :param string value: the new path segment to use
+
+        Example::
+
+            >>> u = URL('http://example.com/foo/')
+            >>> u.add_path_segment('bar')
+            u'http://example.com/foo/bar'
+        """
         segments = self.path_segments() + (value,)
         return self.path_segments(segments)
 
     def has_query_param(self, key):
+        """
+        Test if a given query parameter is present
+
+        :param string key: key to test for
+        """
         return self.query_param(key) is not None
 
     def has_query_params(self, keys):
+        """
+        Test if a given set of query parameters are present
+
+        :param list keys: keys to test for
+        """
         return all([self.has_query_param(k) for k in keys])
 
     def query_param(self, key, value=None, default=None, as_list=False):
         """
         Return a query parameter for the given key
+
+        :param string key: key to look for
+        :param string default: value to return if ``key`` isn't found
+        :param boolean as_list: whether to return the values as a list
+        :param string value: the new query parameter to use
         """
         parse_result = self.query_params()
         if value is not None:
@@ -217,6 +331,11 @@ class URL(object):
         return result[0] if len(result) == 1 else result
 
     def query_params(self, value=None):
+        """
+        Return a dictionary of query params
+
+        :param dict value: new dictionary of values
+        """
         if value is not None:
             return URL._mutate(self, query=urllib.urlencode(value, doseq=True))
         query = '' if self._tuple.query is None else self._tuple.query
